@@ -11,7 +11,7 @@
 from collections import OrderedDict, abc
 from copy import copy
 from datetime import datetime, timedelta
-from typing import Any, List, Optional, Sequence, Set, Union
+from typing import Any, Generic, List, Optional, Sequence, Set, Union
 
 import attr
 import numpy as np
@@ -34,16 +34,6 @@ from hypothesis.internal.validation import (
 from hypothesis.strategies._internal.strategies import Ex, check_strategy
 from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 
-try:
-    from pandas.api.types import is_categorical_dtype
-except ImportError:
-
-    def is_categorical_dtype(dt):
-        if isinstance(dt, np.dtype):
-            return False
-        return dt == "category"
-
-
 try:
     from pandas.core.arrays.integer import IntegerDtype
 except ImportError:
@@ -79,8 +69,8 @@ def elements_and_dtype(elements, dtype, source=None):
                     f"At least one of {prefix}elements or {prefix}dtype must be provided."
                 )
 
-    with check("is_categorical_dtype"):
-        if is_categorical_dtype(dtype):
+    with check("isinstance(dtype, CategoricalDtype)"):
+        if pandas.api.types.CategoricalDtype.is_dtype(dtype):
             raise InvalidArgument(
                 f"{prefix}dtype is categorical, which is currently unsupported"
             )
@@ -356,9 +346,11 @@ def series(
             return pandas.Series(
                 (),
                 index=index,
-                dtype=dtype
-                if dtype is not None
-                else draw(dtype_for_elements_strategy(elements)),
+                dtype=(
+                    dtype
+                    if dtype is not None
+                    else draw(dtype_for_elements_strategy(elements))
+                ),
                 name=draw(name),
             )
 
@@ -366,7 +358,7 @@ def series(
 
 
 @attr.s(slots=True)
-class column:
+class column(Generic[Ex]):
     """Data object for describing a column in a DataFrame.
 
     Arguments:
@@ -383,11 +375,11 @@ class column:
     * unique: If all values in this column should be distinct.
     """
 
-    name = attr.ib(default=None)
-    elements = attr.ib(default=None)
-    dtype = attr.ib(default=None, repr=get_pretty_function_description)
-    fill = attr.ib(default=None)
-    unique = attr.ib(default=False)
+    name: Optional[Union[str, int]] = attr.ib(default=None)
+    elements: Optional[st.SearchStrategy[Ex]] = attr.ib(default=None)
+    dtype: Any = attr.ib(default=None, repr=get_pretty_function_description)
+    fill: Optional[st.SearchStrategy[Ex]] = attr.ib(default=None)
+    unique: bool = attr.ib(default=False)
 
 
 def columns(
@@ -397,7 +389,7 @@ def columns(
     elements: Optional[st.SearchStrategy[Ex]] = None,
     fill: Optional[st.SearchStrategy[Ex]] = None,
     unique: bool = False,
-) -> List[column]:
+) -> List[column[Ex]]:
     """A convenience function for producing a list of :class:`column` objects
     of the same general shape.
 
